     HCOPYRIGHT('(C) Crimson Solutions Ltd, 2005')
     H*===============================================================*/
     H* DOCMDRPG: DOCMD (do command) processor                        */
     H*===============================================================*/
     H*                                                               */
     H* Synopsis:    This program process the DOCMD command.          */
     H*                                                               */
     H* Parameters:  (From command).                                  */
     H*                                                               */
     H*              P0CMD  C(1024) Command to execute                */
     H*                                                               */
     H*              P0OBJ  C(20)   Query definition                  */
     H*                             01-10:  Object name/generic/*ALL  */
     H*                             11-20:  Library/*LIBL/*CURLIB/    */
     H*                                     *ALLUSR/*ALL              */
     H*                                                               */
     H*              P0TYP  C(10)   Type of object                    */
     H*                                                               */
     H* Notes:       This program looks for and expects either a      */
     H*              ##OBJ## or ##LIB## in the command string. If     */
     H*              none is found, it terminates.                    */
     H*              These strings are replaced with the object names */
     H*              found from the list of valid objects generated   */
     H*              in this program.                                 */
     H*                                                               */
     H*===============================================================*/
     H* MAINTENANCE                                                   */
     H*===============================================================*/
     H*                                                               */
     H* Date:        dd/mm/yy                                         */
     H* Amended by:  name                                             */
     H* Mod Code:    xx01                                             */
     H* Description:                                                  */
     H*                                                               */
     F*================================================================
     D*================================================================
      *
      * Command strings
      *
     D @CMD            S             80    DIM(1) CTDATA PERRCD(1)
     D*================================================================
     D                 DS
     D  @MSLEN                 1      4B 0
     D  @MSOFF                 5      8B 0
     D  @MSERR                 9     12B 0
     D P3DATA          DS
     D  P3DSTR                 1      4B 0
     D  P3DCNT                 9     12B 0
     D  P3DSIZ                13     16B 0
     D P5DATA          DS
     D  P5DSTR                 1      4B 0
     D  P5DCNT                 9     12B 0
     D  P5DSIZ                13     16B 0
     D P0OBJ           DS
     D  P0OBJN                 1     10
     D  P0OBJL                11     20
     D QUOOF           DS
     D  QUOOFN                 1     10
     D  QUOOFL                11     20
     D X                              5P 0
      *
      * Binary variables for APIs
      *
     D                 DS
     D  P1USSZ                 1      4B 0
     D  P3RTVO                 5      8B 0
     D  P3RTVL                 9     12B 0
     D  P4PTR                 13     16B 0
     D  P4LEN                 17     20B 0
     D  P5PTR                 21     24B 0
     D  P5LEN                 25     28B 0
     D  P5RTVO                29     32B 0
     D  P5RTVL                33     36B 0
      *
      * Constant definitions
      *
     D @USNA1          C                   CONST('DOCMDUS1  QTEMP     ')
     D @USNA2          C                   CONST('DOCMDUS2  QTEMP     ')
     C*================================================================
     C* Prototyping
     C*================================================================
     C*
     C*
     C* This lot used in prototyping
     C*
     D @MSID           S              7A
     D @MSFIL          S             20A
     D @MSDTA          S            256A
     C*
     C*================================================================
     C                   EXSR      A
     C                   EXSR      B
     C                   EXSR      C
     C*================================================================
     C     A             BEGSR
     C*----------------------------------------------------------------
     C* Initialisation
     C*================================================================
      *
      * Ensure that the command contains either ##OBJ## or ##LIB##
      *
     C                   EVAL      X=%SCAN('##OBJ##':P0CMD)
     C     x             ifeq      *zero
     C                   EVAL      X=%SCAN('##LIB##':P0CMD)
     C     x             ifeq      *zero
     C                   eval      @MSDTA='No object or library specified'
     C                   exsr      SNDMSG
     C                   EXSR      C
     C                   ENDIF
     C                   ENDIF
      *
      * Create a temporary user space
      *
     C                   EXSR      AB
      *
     C                   ENDSR
     C*================================================================
     C     B             BEGSR
     C*----------------------------------------------------------------
     C* Main processing
     C*================================================================
      *
      * Generate the list of queries to the user space
      *
     C                   EXSR      BA
      *
      * Process each of the queries in the list
      *
     C                   EXSR      BB
      *
     C                   ENDSR
      *
     C*================================================================
     C     C             BEGSR
     C*----------------------------------------------------------------
     C* Termination
     C*================================================================
      *
     C                   MOVE      '1'           *INLR
     C                   RETURN
      *
     C                   ENDSR
      *
     C*================================================================
     C     AB            BEGSR
     C*----------------------------------------------------------------
     C*  Set up user space
     C*================================================================
      *
      * Delete any existing user spaces
      *
     C                   CALL      'QCMDEXC'                            97
     C                   PARM                    @CMD(1)
     C                   PARM      80            @CMLEN           15 5
      *
      * Create user spaces
      *
     C                   CALL      'QUSCRTUS'
     C                   PARM      @USNA1        P1USNM           20
     C                   PARM      *BLANKS       P1USAT           10
     C                   PARM      100           P1USSZ
     C                   PARM      *BLANKS       P1USIN            1
     C                   PARM      '*CHANGE'     P1USAU           10
     C                   PARM      *BLANKS       P1USDS           50
      *
     C                   CALL      'QUSCRTUS'
     C                   PARM      @USNA2        P1USNM           20
     C                   PARM      *BLANKS       P1USAT           10
     C                   PARM      100           P1USSZ
     C                   PARM      *BLANKS       P1USIN            1
     C                   PARM      '*CHANGE'     P1USAU           10
     C                   PARM      *BLANKS       P1USDS           50
      *
     C                   ENDSR
      *
     C*================================================================
     C     BA            BEGSR
     C*----------------------------------------------------------------
     C*  List all selected objects to user space
     C*================================================================
      *
     C                   MOVEL(P)  P0OBJN        P2OBNM           20
     C                   MOVE      P0OBJL        P2OBNM
      *
     C                   CALL      'QUSLOBJ'
     C                   PARM      @USNA1        P2USNM           20
     C                   PARM      'OBJL0100'    P2FMT             8
     C                   PARM                    P2OBNM
     C                   PARM      P0TYP         P2OTYP           10
      *
      * Retrieve the number of entries in the user space, and their
      *   length
      *
     C                   CALL      'QUSRTVUS'
     C                   PARM      @USNA1        P3USNM           20
     C                   PARM      125           P3RTVO
     C                   PARM      16            P3RTVL
     C                   PARM                    P3DATA
      *
      * Get the start offset and add 1 to get byte number
      *
     C     1             ADD       P3DSTR        P4PTR
      *
     C                   ENDSR
      *
     C*================================================================
     C     BB            BEGSR
     C*----------------------------------------------------------------
     C*  Process all returned entries in the user space
     C*================================================================
      *
     C     P3DCNT        IFGT      *ZERO
      *
     C     1             DO        P3DCNT
      *
      * Get the next object name from the user space
      *
     C                   CALL      'QUSRTVUS'
     C                   PARM      @USNA1        P4USNM           20
     C                   PARM                    P4PTR
     C                   PARM      20            P4LEN
     C                   PARM      *BLANKS       P4DATA           20
     C                   MOVEL     P4DATA        ZWOBJ            10
     C                   MOVE      P4DATA        ZWLIB            10
      *
      * Now process the actual command for this object
      *
     C                   EXSR      DOCMD1
      *
     C                   ADD       P3DSIZ        P4PTR
      *
     C                   ENDDO
      *
     C                   ENDIF
      *
     C                   ENDSR
      *
      *
     C*================================================================
     C     DOCMD1        BEGSR
     C*
     C* Move in original command
     C*
     C                   MOVEL     *BLANKS       ZWCMD          2048
     C                   MOVEL     P0CMD         ZWCMD
     C*
     C* Scan and replace for ##OBJ##
     C*
     C                   MOVEL     '##OBJ##'     ZWSCAN            7
     C                   Z-ADD     7             ZWS               5 0
     C                   MOVEL     ZWOBJ         ZWREPL           10
     C                   Z-ADD     10            ZWR               5 0
     C                   EXSR      SCNRPL
     C*
     C* Scan and replace for ##LIB##
     C*
     C                   MOVEL     '##LIB##'     ZWSCAN
     C                   MOVEL     ZWLIB         ZWREPL           10
     C                   EXSR      SCNRPL
     C*
     C* Now run the command
     C*
     C                   call      'QCMDEXC'                            99
     C                   parm      ZWCMD         P1CMD          2048
     C                   parm      2048          P1LEN            15 5
     C*
     C* Send error message if can't be done
     C*
     C     *in99         ifeq      *on
     C                   eval      @MSDTA='command failed for object ' +
     C                             ZWOBJ + ' in library ' + ZWLIB
     C                   EXSR      SNDMSG
     C                   endif
     C                   ENDSR
     C*
     C*================================================================
     C*
     C* Subroutine SCNRPL - does a scan and replace in a string
     C*
     C*
     C*================================================================
     C     SCNRPL        BEGSR
     C                   EVAL      X=%SCAN(ZWSCAN: ZWCMD)
     C     x             downe     *zero
     C                   eval      zwcmd=%subst(zwcmd:1:x-1) + zwrepl +
     C                             %SUBST(ZWCMD:X+ZWS)
     C                   EVAL      X=X+ZWS
     C                   EVAL      X=%SCAN(ZWSCAN: ZWCMD: X)
     C                   ENDDO
     C                   ENDSR
      *
     C*================================================================
     C     ZZ            BEGSR
     C*----------------------------------------------------------------
     C* Definitions (not executed)
     C*================================================================
     C     *DTAARA       DEFINE    CSLTOOLKIT    @LICENSE         24
      *
      * Input parameters (from DOCMD command)
      *
     C     *ENTRY        PLIST
     C                   PARM                    P0CMD          1024
     C                   PARM                    P0OBJ
     C                   PARM                    P0TYP            10
      *
     C                   ENDSR
     C*----------------------------------------------------------------
     C* Send program message
     C*================================================================
      *
     C     SNDMSG        BEGSR
      * Next section used for subroutine ZA - send program message
      *
      *
      *
      * @MSID   contains the message ID
      * @MSFIL  contains the qualified message file
      * @MSDTA  contains the message data
      *
     C                   CALL      'QMHSNDPM'
     C                   PARM                    @MSID
     C                   PARM                    @MSFIL
     C                   PARM                    @MSDTA
     C                   PARM      256           @MSLEN
     C                   PARM      '*INFO'       @MSTYP           10
     C                   PARM      '*'           @MSPGM           10
     C                   PARM      1             @MSOFF
     C                   PARM      *BLANKS       @MSGXX            1
     C                   PARM      0             @MSERR
      *
     C                   ENDSR
** @CMD: Command strings
DLTUSRSPC QTEMP/DOCMDUS*
